/* $Id: config.c 26 2023-04-22 12:07:21Z nishi $ */
#include <swinehub/config.h>

#include <swinehub/util.h>

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

struct swinehub_config* swinehub_parse_config(const char* path){
	if(access(path, F_OK) != 0){
		fprintf(stderr, "SwineHub: WARN! Config `%s` not found. Using default configuration (Prefix = `%s`, Port = `%d`, Server = `%s`, Path = `%s`, CSSPath = `%s`, IconPath = `%s`, ItWorks = `%d`, ScriptPath = `%s`, CachePath = `%s`, SVN = `%s`, SVNAdmin = `%s`, Htpasswd = `%s`, Signup = `%s`, Title = `%s`)\n", path, SWINEHUB_DEFAULT_PREFIX, SWINEHUB_DEFAULT_PORT, SWINEHUB_DEFAULT_SERVER, SWINEHUB_DEFAULT_PATH, SWINEHUB_DEFAULT_CSSPATH, SWINEHUB_DEFAULT_ICONPATH, SWINEHUB_DEFAULT_ITWORKS, SWINEHUB_DEFAULT_SCRIPTPATH, SWINEHUB_DEFAULT_CACHEPATH, SWINEHUB_DEFAULT_SVN, SWINEHUB_DEFAULT_SVNADMIN, SWINEHUB_DEFAULT_HTPASSWD, SWINEHUB_DEFAULT_SIGNUP, SWINEHUB_DEFAULT_TITLE);
	}
	struct swinehub_config* config = malloc(sizeof(*config));
	config->prefix = strdup(SWINEHUB_DEFAULT_PREFIX);
	config->port = SWINEHUB_DEFAULT_PORT;
	config->server = strdup(SWINEHUB_DEFAULT_SERVER);
	config->path = strdup(SWINEHUB_DEFAULT_PATH);
	config->csspath = strdup(SWINEHUB_DEFAULT_CSSPATH);
	config->iconpath = strdup(SWINEHUB_DEFAULT_ICONPATH);
	config->scriptpath = strdup(SWINEHUB_DEFAULT_SCRIPTPATH);
	config->itworks = SWINEHUB_DEFAULT_ITWORKS;
	config->cachepath = strdup(SWINEHUB_DEFAULT_CACHEPATH);
	config->svn = strdup(SWINEHUB_DEFAULT_SVN);
	config->svnadmin = strdup(SWINEHUB_DEFAULT_SVNADMIN);
	config->htpasswd = strdup(SWINEHUB_DEFAULT_HTPASSWD);
	config->signup = SWINEHUB_DEFAULT_SIGNUP;
	config->title = strdup(SWINEHUB_DEFAULT_TITLE);
	FILE* f = fopen(path, "r");
	if(f != NULL){
		char* buf = malloc(2);
		buf[1] = 0;
		char* str = malloc(1);
		str[0] = 0;
		while(1){
			fread(buf, 1, 1, f);
			char ch = buf[0];
			if(ch == '\n' || feof(f)){
				if(strlen(str) != 0){
					char* key = str;
					int i;
					for(i = 0; key[i] != 0 && key[i] != '='; i++);
					if(key[i] == '='){
						key[i] = 0;
						char* value = key + i + 1;
						if(strcmp(key, "ItWorks") == 0){
							if(strcmp(value, "1") == 0 || strcmp(value, "Y") == 0 || strcmp(value, "Yes") == 0 || strcmp(value, "T") == 0){
								config->itworks = 1;
							}else if(strcmp(value, "0") == 0 || strcmp(value, "N") == 0 || strcmp(value, "No") == 0 || strcmp(value, "T") == 0){
								config->itworks = 0;
							}
						}else if(strcmp(key, "Signup") == 0){
							if(strcmp(value, "1") == 0 || strcmp(value, "Y") == 0 || strcmp(value, "Yes") == 0 || strcmp(value, "T") == 0){
								config->signup = 1;
							}else if(strcmp(value, "0") == 0 || strcmp(value, "N") == 0 || strcmp(value, "No") == 0 || strcmp(value, "T") == 0){
								config->signup = 0;
							}
						}else if(strcmp(key, "Prefix") == 0){
							free(config->prefix);
							config->prefix = strdup(value);
						}else if(strcmp(key, "Path") == 0){
							free(config->path);
							config->path = strdup(value);
						}else if(strcmp(key, "CSSPath") == 0){
							free(config->csspath);
							config->csspath = strdup(value);
						}else if(strcmp(key, "IconPath") == 0){
							free(config->iconpath);
							config->iconpath = strdup(value);
						}else if(strcmp(key, "ScriptPath") == 0){
							free(config->scriptpath);
							config->scriptpath = strdup(value);
						}else if(strcmp(key, "CachePath") == 0){
							free(config->cachepath);
							config->cachepath = strdup(value);
						}else if(strcmp(key, "SVN") == 0){
							free(config->svn);
							config->svn = strdup(value);
						}else if(strcmp(key, "SVNAdmin") == 0){
							free(config->svnadmin);
							config->svnadmin = strdup(value);
						}else if(strcmp(key, "Htpasswd") == 0){
							free(config->htpasswd);
							config->htpasswd = strdup(value);
						}else if(strcmp(key, "Server") == 0){
							free(config->server);
							config->server = strdup(value);
						}else if(strcmp(key, "Port") == 0){
							config->port = atoi(value);
						}else if(strcmp(key, "Title") == 0){
							free(config->title);
							config->title = strdup(value);
						}
					}
				}
				free(str);
				str = malloc(1);
				str[0] = 0;
				if(feof(f)) break;
			}else{
				char* tmp = str;
				str = sstrcat(tmp, buf);
				free(tmp);
			}
		}
		free(str);
		free(buf);
		fclose(f);
	}
	return config;
}

void swinehub_free_config(struct swinehub_config* config){
	free(config->prefix);
	free(config->server);
	free(config->path);
	free(config->csspath);
	free(config->iconpath);
	free(config->scriptpath);
	free(config->cachepath);
	free(config->svn);
	free(config->svnadmin);
	free(config->htpasswd);
	free(config->title);
	free(config);
}
